"""A modification workflow for migrating an ImportedOpengear to an Opengear subscription."""

from orchestrator.targets import Target
from orchestrator.types import State, UUIDstr
from orchestrator.workflow import StepList, done, init, step, workflow
from orchestrator.workflows.steps import resync, store_process_subscription, unsync
from orchestrator.workflows.utils import wrap_modify_initial_input_form

from gso.products import ProductName
from gso.products.product_types.opengear import ImportedOpengear, Opengear
from gso.services.subscriptions import get_product_id_by_name


@step("Create new Opengear subscription")
def import_opengear_subscription(subscription_id: UUIDstr) -> State:
    """Take an ImportedOpengear subscription, and turn it into an Opengear subscription."""
    old_opengear = ImportedOpengear.from_subscription(subscription_id)
    product_id = get_product_id_by_name(ProductName.OPENGEAR)
    new_subscription = Opengear.from_other_product(old_opengear, product_id)  # type: ignore[arg-type]

    return {"subscription": new_subscription}


@workflow("Import Opengear", target=Target.MODIFY, initial_input_form=wrap_modify_initial_input_form(None))
def import_opengear() -> StepList:
    """Modify an ImportedOpengear subscription into an Opengear subscription to complete the import."""
    return init >> store_process_subscription(Target.MODIFY) >> unsync >> import_opengear_subscription >> resync >> done
